@@ -96,8 +96,18 @@ void Bitmap::SetDimensions(const unsigned int *dims)
 {
   assert( NumberOfDimensions );
   //assert( Dimensions.empty() );
+#if 0
   Dimensions = std::vector<unsigned int>(dims,
     dims+NumberOfDimensions);
+#else
+  assert( Dimensions.size() == 3 );
+  Dimensions[0] = dims[0];
+  Dimensions[1] = dims[1];
+  if( NumberOfDimensions == 2 )
+    Dimensions[2] = 1;
+  else
+    Dimensions[2] = dims[2];
+#endif
 }
 
 void Bitmap::SetDimension(unsigned int idx, unsigned int dim)
@@ -122,7 +132,6 @@ unsigned int Bitmap::GetPlanarConfiguration() const
 {
   if( PlanarConfiguration && PF.GetSamplesPerPixel() != 3 )
     {
-    assert(0);
     // LEADTOOLS_FLOWERS-8-PAL-RLE.dcm
     // User specify PlanarConfiguration whereas SamplesPerPixel != 3
     gdcmWarningMacro(
@@ -137,9 +146,40 @@ void Bitmap::SetPlanarConfiguration(unsigned int pc)
 {
   // precondition
   assert( pc == 0 || pc == 1 );
-  // LEADTOOLS_FLOWERS-8-MONO2-Uncompressed.dcm
-  if( pc ) assert( PF.GetSamplesPerPixel() == 3 ); // Please set PixelFormat first
   PlanarConfiguration = pc;
+  if( pc )
+    {
+    // LEADTOOLS_FLOWERS-8-MONO2-Uncompressed.dcm
+    if( PF.GetSamplesPerPixel() != 3 ) // Please set PixelFormat first
+      {
+      gdcmWarningMacro( "Cant have Planar Configuration in non RGB input. Discarding" );
+      PlanarConfiguration = 0;
+      }
+    const TransferSyntax &ts = GetTransferSyntax();
+    if(  ts == TransferSyntax::JPEGBaselineProcess1
+      || ts == TransferSyntax::JPEGExtendedProcess2_4
+      || ts == TransferSyntax::JPEGExtendedProcess3_5
+      || ts == TransferSyntax::JPEGSpectralSelectionProcess6_8
+      || ts == TransferSyntax::JPEGFullProgressionProcess10_12
+      || ts == TransferSyntax::JPEGLosslessProcess14
+      || ts == TransferSyntax::JPEGLosslessProcess14_1
+      || ts == TransferSyntax::JPEGLSLossless
+      || ts == TransferSyntax::JPEGLSNearLossless
+      || ts == TransferSyntax::JPEG2000Lossless
+      || ts == TransferSyntax::JPEG2000
+      || ts == TransferSyntax::JPIPReferenced
+      || ts == TransferSyntax::RLELossless // FIXME internally GDCM produce per-pixel output
+    )
+      {
+      // PS 3.6 - 2011 8.2.4 JPEG 2000 IMAGE COMPRESSION
+      // The value of Planar Configuration (0028,0006) is irrelevant since the
+      // manner of encoding components is specified in the JPEG 2000 standard,
+      // hence it shall be set to 0.
+      // By extension, this behavior has been applied also to JPEG and JPEG-LS
+      gdcmWarningMacro( "Cant have Planar Configuration in JPEG/JPEG-LS/JPEG 2000. Discarding" );
+      PlanarConfiguration = 0;
+      }
+    }
   // \postcondition
   assert( PlanarConfiguration == 0 || PlanarConfiguration == 1 );
 }
@@ -178,7 +218,7 @@ bool Bitmap::GetBuffer(char *buffer) const
     // contains a compressed Icon Sequence, one has to guess this is lossless jpeg...
 #ifdef MDEBUG
     const SequenceOfFragments *sqf = PixelData.GetSequenceOfFragments();
-    std::ofstream os( "/tmp/kodak.ljpeg");
+    std::ofstream os( "/tmp/kodak.ljpeg", std::ios::binary);
     sqf->WriteBuffer( os );
 #endif
     gdcmWarningMacro( "Compressed Icon are not support for now" );
@@ -245,9 +285,12 @@ unsigned long Bitmap::GetBufferLength() const
   else if( PF == PixelFormat::SINGLEBIT )
     {
     assert( PF.GetSamplesPerPixel() == 1 );
-    unsigned int save = mul;
-    save /= 8;
-    assert( save * 8 == mul );
+    const size_t bytesPerRow = Dimensions[0] / 8 + (Dimensions[0] % 8 != 0 ? 1 : 0);
+    unsigned int save = bytesPerRow * Dimensions[1];
+    if( NumberOfDimensions > 2 )
+      save *= Dimensions[2];
+    if(Dimensions[0] % 8 == 0 )
+      assert( save * 8 == mul );
     mul = save;
     }
   else if( PF.GetBitsAllocated() % 8 != 0 )
@@ -298,7 +341,7 @@ bool Bitmap::TryRAWCodec(char *buffer, bool &lossyflag) const
     codec.SetLUT( GetLUT() );
     codec.SetPixelFormat( GetPixelFormat() );
     codec.SetNeedByteSwap( GetNeedByteSwap() );
-    codec.SetNeedOverlayCleanup( AreOverlaysInPixelData() );
+    codec.SetNeedOverlayCleanup( AreOverlaysInPixelData() || UnusedBitsPresentInPixelData() );
     DataElement out;
     //bool r = codec.Decode(PixelData, out);
     bool r = codec.DecodeBytes(bv->GetPointer(), bv->GetLength(),
@@ -319,7 +362,7 @@ bool Bitmap::TryRAWCodec(char *buffer, bool &lossyflag) const
 #if 0
     if ( GetPixelFormat() != codec.GetPixelFormat() )
       {
-      gdcm::Bitmap *i = (gdcm::Bitmap*)this;
+      Bitmap *i = (Bitmap*)this;
       i->SetPixelFormat( codec.GetPixelFormat() );
       }
 #endif
@@ -348,7 +391,7 @@ bool Bitmap::TryJPEGCodec(char *buffer, bool &lossyflag) const
       if( !sf ) return false;
       const Fragment &frag = sf->GetFragment(0);
       const ByteValue &bv2 = dynamic_cast<const ByteValue&>(frag.GetValue());
-      gdcm::PixelFormat pf = GetPixelFormat(); // gdcm::PixelFormat::UINT8;
+      PixelFormat pf = GetPixelFormat(); // PixelFormat::UINT8;
       codec.SetPixelFormat( pf );
 
       std::stringstream ss;
@@ -362,10 +405,16 @@ bool Bitmap::TryJPEGCodec(char *buffer, bool &lossyflag) const
 #if 0
       if( codec.GetPixelFormat() != GetPixelFormat() )
         {
-        gdcm::Bitmap *i = (gdcm::Bitmap*)this;
+        Bitmap *i = (Bitmap*)this;
         i->SetPixelFormat( codec.GetPixelFormat() );
         }
 #endif
+      if( GetDimensions()[0] != codec.GetDimensions()[0]
+      || GetDimensions()[1] != codec.GetDimensions()[1] )
+{
+      gdcmWarningMacro( "dimension mismatch for JPEG" );
+	((Bitmap*)this)->SetDimensions( codec.GetDimensions() ); //JPEGNote_bogus.dcm
+}
 
       return true;
       }
@@ -375,10 +424,12 @@ bool Bitmap::TryJPEGCodec(char *buffer, bool &lossyflag) const
   if( codec.CanDecode( ts ) )
     {
     unsigned long len = GetBufferLength();
+    codec.SetNumberOfDimensions( GetNumberOfDimensions() );
+    codec.SetDimensions( GetDimensions() );
     codec.SetPlanarConfiguration( GetPlanarConfiguration() );
     codec.SetPhotometricInterpretation( GetPhotometricInterpretation() );
     codec.SetPixelFormat( GetPixelFormat() );
-    codec.SetNeedOverlayCleanup( AreOverlaysInPixelData() );
+    codec.SetNeedOverlayCleanup( AreOverlaysInPixelData() || UnusedBitsPresentInPixelData() );
     DataElement out;
     bool r = codec.Decode(PixelData, out);
     // PHILIPS_Gyroscan-12-MONO2-Jpeg_Lossless.dcm
@@ -390,7 +441,7 @@ bool Bitmap::TryJPEGCodec(char *buffer, bool &lossyflag) const
     // Did PI change or not ?
     if ( GetPlanarConfiguration() != codec.GetPlanarConfiguration() )
       {
-      gdcm::Bitmap *i = (gdcm::Bitmap*)this; (void)i;
+      Bitmap *i = (Bitmap*)this; (void)i;
       //i->SetPlanarConfiguration( codec.GetPlanarConfiguration() );
       }
     // I cannot re-activate the following since I would loose the palette color information
@@ -399,7 +450,7 @@ bool Bitmap::TryJPEGCodec(char *buffer, bool &lossyflag) const
     //  {
     //  // HACK
     //  // YBRisGray.dcm
-    //  gdcm::Bitmap *i = (gdcm::Bitmap*)this;
+    //  Bitmap *i = (Bitmap*)this;
     //  i->SetPhotometricInterpretation( codec.GetPhotometricInterpretation() );
     //  }
 #if 1
@@ -411,14 +462,14 @@ bool Bitmap::TryJPEGCodec(char *buffer, bool &lossyflag) const
 //      assert( GetPixelFormat().GetBitsStored() ==
 //        codec.GetPixelFormat().GetBitsStored() );
       assert( GetPixelFormat().GetBitsAllocated() == 12 );
-      gdcm::Bitmap *i = (gdcm::Bitmap*)this;
+      Bitmap *i = (Bitmap*)this;
       i->SetPixelFormat( codec.GetPixelFormat() );
       }
 #endif
     //if ( GetPhotometricInterpretation() == PhotometricInterpretation::YBR_FULL_422
     //|| GetPhotometricInterpretation() == PhotometricInterpretation::YBR_FULL )
     //  {
-    //  gdcm::Bitmap *i = (gdcm::Bitmap*)this;
+    //  Bitmap *i = (Bitmap*)this;
     //  i->SetPhotometricInterpretation( PhotometricInterpretation::RGB );
     //  }
     const ByteValue *outbv = out.GetByteValue();
@@ -426,6 +477,11 @@ bool Bitmap::TryJPEGCodec(char *buffer, bool &lossyflag) const
     unsigned long check = outbv->GetLength();  // FIXME
     (void)check;
     // DermaColorLossLess.dcm has a len of 63531, but DICOM will give us: 63532 ...
+    if( len > outbv->GetLength() )
+      {
+      gdcmErrorMacro( "Impossible length: " << len << " should be (max): " << outbv->GetLength() );
+      return false;
+      }
     assert( len <= outbv->GetLength() );
     if(buffer) memcpy(buffer, outbv->GetPointer(), len /*outbv->GetLength()*/ );  // FIXME
 
@@ -449,7 +505,7 @@ bool Bitmap::TryJPEGCodec2(std::ostream &os) const
     codec.SetPlanarConfiguration( GetPlanarConfiguration() );
     codec.SetPhotometricInterpretation( GetPhotometricInterpretation() );
     codec.SetPixelFormat( GetPixelFormat() );
-    codec.SetNeedOverlayCleanup( AreOverlaysInPixelData() );
+    codec.SetNeedOverlayCleanup( AreOverlaysInPixelData() || UnusedBitsPresentInPixelData() );
     DataElement out;
     bool r = codec.Code(PixelData, out);
     // PHILIPS_Gyroscan-12-MONO2-Jpeg_Lossless.dcm
@@ -462,7 +518,7 @@ bool Bitmap::TryJPEGCodec2(std::ostream &os) const
     if ( GetPhotometricInterpretation() != codec.GetPhotometricInterpretation() )
       {
       // HACK
-      //gdcm::Bitmap *i = (gdcm::Bitmap*)this;
+      //Bitmap *i = (Bitmap*)this;
       //i->SetPhotometricInterpretation( codec.GetPhotometricInterpretation() );
       }
     const ByteValue *outbv = out.GetByteValue();
@@ -492,12 +548,18 @@ bool Bitmap::TryPVRGCodec(char *buffer, bool &lossyflag) const
     //codec.SetNumberOfDimensions( GetNumberOfDimensions() );
     codec.SetPlanarConfiguration( GetPlanarConfiguration() );
     codec.SetPhotometricInterpretation( GetPhotometricInterpretation() );
-    codec.SetNeedOverlayCleanup( AreOverlaysInPixelData() );
+    codec.SetNeedOverlayCleanup( AreOverlaysInPixelData() || UnusedBitsPresentInPixelData() );
     codec.SetDimensions( GetDimensions() );
     DataElement out;
     bool r = codec.Decode(PixelData, out);
     if(!r) return false;
+    codec.SetLossyFlag( ts.IsLossy() );
     assert( r );
+    if ( GetPlanarConfiguration() != codec.GetPlanarConfiguration() )
+      {
+      Bitmap *i = (Bitmap*)this;
+      i->PlanarConfiguration = codec.GetPlanarConfiguration();
+      }
     const ByteValue *outbv = out.GetByteValue();
     assert( outbv );
     unsigned long check = outbv->GetLength();  // FIXME
@@ -526,7 +588,7 @@ bool Bitmap::TryKAKADUCodec(char *buffer, bool &lossyflag) const
     codec.SetNumberOfDimensions( GetNumberOfDimensions() );
     codec.SetPlanarConfiguration( GetPlanarConfiguration() );
     codec.SetPhotometricInterpretation( GetPhotometricInterpretation() );
-    codec.SetNeedOverlayCleanup( AreOverlaysInPixelData() );
+    codec.SetNeedOverlayCleanup( AreOverlaysInPixelData() || UnusedBitsPresentInPixelData() );
     codec.SetDimensions( GetDimensions() );
     DataElement out;
     bool r = codec.Decode(PixelData, out);
@@ -580,7 +642,7 @@ bool Bitmap::TryJPEGLSCodec(char *buffer, bool &lossyflag) const
 // therefore -rightfully- declared as 16 bits...
       if( codec.GetPixelFormat() != GetPixelFormat() )
         {
-        gdcm::Bitmap *i = (gdcm::Bitmap*)this;
+        Bitmap *i = (Bitmap*)this;
         i->SetPixelFormat( codec.GetPixelFormat() );
         }
 #endif
@@ -599,7 +661,7 @@ bool Bitmap::TryJPEGLSCodec(char *buffer, bool &lossyflag) const
     codec.SetNumberOfDimensions( GetNumberOfDimensions() );
     codec.SetPlanarConfiguration( GetPlanarConfiguration() );
     codec.SetPhotometricInterpretation( GetPhotometricInterpretation() );
-    codec.SetNeedOverlayCleanup( AreOverlaysInPixelData() );
+    codec.SetNeedOverlayCleanup( AreOverlaysInPixelData() || UnusedBitsPresentInPixelData() );
     codec.SetDimensions( GetDimensions() );
     DataElement out;
     bool r = codec.Decode(PixelData, out);
@@ -672,7 +734,7 @@ bool Bitmap::TryJPEG2000Codec(char *buffer, bool &lossyflag) const
         {
         // Because J2K support the full spectrum I do not see any issue
         // with the following:
-        gdcm::Bitmap *i = (gdcm::Bitmap*)this;
+        Bitmap *i = (Bitmap*)this;
         i->SetPixelFormat( codec.GetPixelFormat() );
         }
 #else
@@ -688,13 +750,20 @@ bool Bitmap::TryJPEG2000Codec(char *buffer, bool &lossyflag) const
             {
             if( cpf.GetBitsStored() < pf.GetBitsStored() )
               {
-              gdcm::Bitmap *i = (gdcm::Bitmap*)this;
+              Bitmap *i = (Bitmap*)this;
               gdcmWarningMacro( "Encapsulated stream has fewer bits actually stored on disk. correcting." );
               i->GetPixelFormat().SetBitsStored( cpf.GetBitsStored() );
               }
             }
           }
         }
+      else
+        {
+        // SC16BitsAllocated_8BitsStoredJ2K.dcm
+        gdcmWarningMacro( "Bits Allocated are different. This is pretty bad using info from codestream" );
+        Bitmap *i = (Bitmap*)this;
+        i->SetPixelFormat( codec.GetPixelFormat() );
+        }
 #endif
 
       return true;
@@ -709,7 +778,7 @@ bool Bitmap::TryJPEG2000Codec(char *buffer, bool &lossyflag) const
     codec.SetNumberOfDimensions( GetNumberOfDimensions() );
     codec.SetPlanarConfiguration( GetPlanarConfiguration() );
     codec.SetPhotometricInterpretation( GetPhotometricInterpretation() );
-    codec.SetNeedOverlayCleanup( AreOverlaysInPixelData() );
+    codec.SetNeedOverlayCleanup( AreOverlaysInPixelData() || UnusedBitsPresentInPixelData() );
     codec.SetDimensions( GetDimensions() );
     DataElement out;
     bool r = codec.Decode(PixelData, out);
@@ -732,7 +801,7 @@ bool Bitmap::TryJPEG2000Codec(char *buffer, bool &lossyflag) const
 #if 0
     if( codec.GetPixelFormat() != GetPixelFormat() )
       {
-      gdcm::Bitmap *i = (gdcm::Bitmap*)this;
+      Bitmap *i = (Bitmap*)this;
       i->SetPixelFormat( codec.GetPixelFormat() );
       }
 #else
@@ -748,7 +817,7 @@ bool Bitmap::TryJPEG2000Codec(char *buffer, bool &lossyflag) const
             {
             if( cpf.GetBitsStored() < pf.GetBitsStored() )
               {
-              gdcm::Bitmap *i = (gdcm::Bitmap*)this;
+              Bitmap *i = (Bitmap*)this;
               gdcmWarningMacro( "Encapsulated stream has fewer bits actually stored on disk. correcting." );
               i->GetPixelFormat().SetBitsStored( cpf.GetBitsStored() );
               }
@@ -776,7 +845,7 @@ bool Bitmap::TryJPEG2000Codec2(std::ostream &os) const
     codec.SetNumberOfDimensions( GetNumberOfDimensions() );
     codec.SetPlanarConfiguration( GetPlanarConfiguration() );
     codec.SetPhotometricInterpretation( GetPhotometricInterpretation() );
-    codec.SetNeedOverlayCleanup( AreOverlaysInPixelData() );
+    codec.SetNeedOverlayCleanup( AreOverlaysInPixelData() || UnusedBitsPresentInPixelData() );
     DataElement out;
     bool r = codec.Code(PixelData, out);
     assert( r );
@@ -807,7 +876,7 @@ bool Bitmap::TryRLECodec(char *buffer, bool &lossyflag ) const
     codec.SetPhotometricInterpretation( GetPhotometricInterpretation() );
     codec.SetPixelFormat( GetPixelFormat() );
     codec.SetLUT( GetLUT() );
-    codec.SetNeedOverlayCleanup( AreOverlaysInPixelData() );
+    codec.SetNeedOverlayCleanup( AreOverlaysInPixelData() || UnusedBitsPresentInPixelData() );
     codec.SetBufferLength( len );
     DataElement out;
     bool r = codec.Decode(PixelData, out);
